#include <gst/gst.h>
#include <stdlib.h>
#include <fcntl.h>
#include <unistd.h>

#ifdef OS_C8K
#include <osc8k.h>
#endif

#include "ss_iac.h"
#include "ss_gst.h"


#define VOC_DEFAULT_FILE "/mnt/doc/vroot/opt/voicecall/pixmap/1.mp3"
#define SMS_DEFAULT_FILE "/mnt/doc/vroot/opt/voicecall/pixmap/1.mp3"
#define ALL_DEFAULT_FILE "/mnt/doc/vroot/opt/voicecall/pixmap/1.mp3"


extern gboolean    is_playing;

#ifdef OS_C8K
static gboolean    shake_on      = FALSE;
static guint       shake_id      = 0;
#endif

static GstElement  *volume       = NULL;
static GstElement  *bin          = NULL;
static GMainLoop   *gst_loop     = NULL;
static gdouble     remute_volume = 0;


static void ss_gst_message_received (GstBus *bus, GstMessage *message, GstPipeline * pipeline)
{
  const GstStructure *s;

  s = gst_message_get_structure (message);
  g_print ("message from \"%s\" (%s): ",
           GST_STR_NULL (GST_ELEMENT_NAME (GST_MESSAGE_SRC (message))),
           gst_message_type_get_name (GST_MESSAGE_TYPE (message)));
  if (s) {
    gchar *sstr;

    sstr = gst_structure_to_string (s);
    g_print ("%s\n", sstr);
    g_free (sstr);
  } else {
    g_print ("no message details\n");
  }
}


static gboolean ss_gst_bus_watcher (GstBus *bus, GstMessage *message, gpointer data)
{
  SSMessParser *controller;
  controller = (SSMessParser *) data;
  
  switch (GST_MESSAGE_TYPE (message)) {
    case GST_MESSAGE_EOS: {
      if (controller->applicant == APP_VOC) {
        g_print ("APP_VOC\n");
        gst_element_set_state (bin, GST_STATE_NULL);
        gst_element_set_state (bin, GST_STATE_PLAYING);
      }
      else {
        g_print ("End of file: %s: %d\n", __FUNCTION__, __LINE__);
        g_main_loop_quit (gst_loop);
      }
      break;
    }
    case GST_MESSAGE_ERROR: {
      gchar *debug;
      GError *err;
      
      gst_message_parse_error (message, &err, &debug);
      g_free (debug);
      
      g_print ("Error: %s\n", err->message);
      g_error_free (err);
      
      g_main_loop_quit (gst_loop);
      break;
    }
    default:
      break;
  }
  
  return TRUE;
}


static gchar *ss_gst_get_default_path (SSMessParser *controller)
{
  gchar *default_file;
  if (controller->applicant < 0) {
    g_print ("Unknow applicant.\n");
    default_file = ALL_DEFAULT_FILE;
  }
  else {
    switch (controller->applicant) {
      case APP_VOC: {
        default_file = VOC_DEFAULT_FILE;
        break;
      }
      case APP_SMS: {
        default_file = SMS_DEFAULT_FILE;
        break;
      }
      default: {
        default_file = "/mnt/doc/bin/indie--London.mp3";
        break;
      }
    }
  }
  
  return default_file;
}


void ss_gst_volume_set_shift (gdouble shift)
{
  if (!volume) {
    g_print ("Can't get the volume element from the pipeline.\n");
    return;
  }
  
  else {
    gdouble temp_volume;
    g_object_get (G_OBJECT (volume), "volume", &temp_volume, NULL);
    temp_volume += shift;
    if (temp_volume < 0)
      temp_volume = 0;
    else if (temp_volume > 4)
      temp_volume = 4;
    g_object_set (G_OBJECT (volume), "volume", temp_volume, NULL);
  }
  
  g_object_get (G_OBJECT (volume), "volume", &remute_volume, NULL);
}

void ss_gst_volume_set_num (gdouble level)
{
  if (!volume) {
    g_printf ("Can't get the volume element from the pipeline.\n");
    return;
  }
  
  else {
    g_object_get (G_OBJECT (volume), "volume", &remute_volume, NULL);
    if (level < 0)
      level = 0;
    else if (level > 4)
      level = 4;
    g_object_set (G_OBJECT (volume), "volume", level, NULL);
  }
  
  gdouble temp_volume;
  g_object_get (G_OBJECT (volume), "volume", &temp_volume, NULL);
  if (temp_volume != 0)
    remute_volume = temp_volume;
}

void ss_gst_volume_get (gdouble * level)
{
  if (!volume)
    g_print ("Can't get the volume element from the pipeline.\n");
  
  else
    g_object_get (G_OBJECT (volume), "volume", level, NULL);
}

void ss_gst_remute ()
{
  if (!volume) {
    g_printf ("Can't get the volume element from the pipeline.\n");
    return;
  }
  g_object_set (G_OBJECT (volume), "volume", remute_volume, NULL);
}

void ss_gst_stop ()
{
  g_main_loop_quit (gst_loop);
}

void ss_gst_pause ()
{
  gst_element_set_state (bin, GST_STATE_PAUSED);
}

void ss_gst_continue ()
{
  gst_element_set_state (bin, GST_STATE_PLAYING);
}

#ifdef OS_C8K
static gboolean cb_shake_control (gpointer data)
{
 	if (shake_on) {
 	  shake_on = FALSE;
    os_vibrate_off ();
	}
	else {
  	shake_on = TRUE;
    os_vibrate_on ();
 	}

  return is_playing;
}

static void ss_shake ()
{
  gchar *tmp;
  gchar *vibrator;
  
  tmp = getenv ("GPE_PATH");
  vibrator = g_strdup_printf ("%s/bin/vibrator", tmp);
  
  gboolean shake_model = FALSE;
  gboolean shake_on    = FALSE;
  
  shake_model = g_file_test ("/tmp/vibrate", G_FILE_TEST_EXISTS);
  if (shake_model) {
    shake_id = g_timeout_add (2000, 
									 (GSourceFunc)(cb_shake_control),
										NULL);
  }
}
#endif


gpointer ss_gst_player (gpointer data)
{
  GstElement   *filesrc;
  GstElement   *decoder;
  GstElement   *converter;
  GstElement   *audiosink;
  
  GstBus       *bus;
  GError       *error = NULL;
  SSMessParser *controller;
  gchar        *filename;
  gint          fd;
  
  controller = (SSMessParser *)data;
  if (controller->value.location) {
    filename = (gchar *) controller->value.location;
    fd = open (filename, O_RDONLY);
    if (fd < 0)
      filename = ss_gst_get_default_path (controller);
  }
  else
    filename = ss_gst_get_default_path (controller);

#ifdef OS_C8K
  ss_shake ();
#endif
 
  bin = gst_bin_new ("mp3");
  
  filesrc = gst_element_factory_make ("filesrc", "my_filesrc");
  g_object_set (G_OBJECT (filesrc), "location", filename, NULL);
  decoder = gst_element_factory_make ("mad", "decoder");
  converter = gst_element_factory_make ("audioconvert", "convert");
  volume = gst_element_factory_make ("volume", "volume");
  g_object_set (G_OBJECT (volume), "volume", 0.5, NULL);
  audiosink = gst_element_factory_make ("esdsink", "audiosink");
  
  gst_bin_add_many (GST_BIN (bin), filesrc, decoder, converter, volume, audiosink, NULL);
  gst_element_link_many (filesrc, decoder, converter, volume, audiosink, NULL);
  g_print ("%s: %d, bin content:%u, bin location: %u,\n", __FUNCTION__, __LINE__, bin, &bin);
  
  gst_loop = g_main_loop_new (NULL, FALSE);
	bus = gst_bus_new ();
	gst_element_set_bus (GST_ELEMENT (bin), bus);
  gst_bus_add_watch (bus, ss_gst_bus_watcher, (gpointer) controller);
  
  gst_element_set_state (bin, GST_STATE_PLAYING);
  g_main_loop_run (gst_loop);
  gst_element_set_state (bin, GST_STATE_NULL);
  
  gst_object_unref (bin);
  bin = NULL;
  g_main_loop_unref (gst_loop);
  gst_loop = NULL;
  
#ifdef OS_C8K
  if (shake_id != 0) {
		g_source_remove (shake_id);
    shake_id = 0;
    os_vibrate_off ();
	}
#endif

  is_playing = FALSE;
}




