#include <glib.h>
#include <iac.h>
#include <string.h>
#include <stdlib.h>

#include "ss_iac.h"
#include "ss_gst.h"

static iac_context_t *context    = NULL;
static SSMessParser  *controller = NULL;

gboolean is_playing              = FALSE;


static SSMessParser *ss_controller_new (void)
{
  SSMessParser *controller;
  controller = (SSMessParser *)g_malloc0 (sizeof (SSMessParser));
  
  return controller;
}


static void ss_controller_free (SSMessParser *controller)
{
  if (controller->type == CON_LOCATION)
  {
    if (controller->value.location) {
      g_free (controller->value.location);
      controller->value.location = NULL;
    }
  }
}


static Applicant ss_parser_get_applicant (const gchar * str_user)
{
//   g_print ("%s: %d\n", __FUNCTION__, __LINE__);
  Applicant applicant;
  
  if (!strcmp (str_user, "voc")) {
    applicant = APP_VOC;
    goto success;
  }
  
  if (!strcmp (str_user, "sms")) {
    applicant = APP_SMS;
    goto success;
  }
  return -1;
  
  success: {
//     g_print ("%s: %d\n", __FUNCTION__, __LINE__);
    return applicant;
  }

}


static ControlType ss_parser_get_control_type (const gchar * str_type)
{
  g_print ("%s: %d\n", __FUNCTION__, __LINE__);
  ControlType type;
  
  if (!strcmp (str_type, "location")) {
    g_print ("%s: %d location\n", __FUNCTION__, __LINE__);
    type = CON_LOCATION;
    goto success;
  }

  if (!strcmp (str_type, "mute")) {
    type = CON_MUTE;
    goto success;
  }
  
  if (!strcmp (str_type, "stop")) {
    g_print ("%s: %d stop\n", __FUNCTION__, __LINE__);
    type = CON_STOP;
    goto success;
  }
  
  if (!strcmp (str_type, "volume")) {
    type = CON_VOLUME;
    goto success;
  }
  
  if (!strcmp (str_type, "remute")) {
    type = CON_REMUTE;
    goto success;
  }

  if (!strcmp (str_type, "pause")) {
    type = CON_PAUSE;
    goto success;
  }

  if (!strcmp (str_type, "continue")) {
    type = CON_CONTINUE;
    goto success;
  }
  
  success: {
      g_print ("%s: %d\n", __FUNCTION__, __LINE__);
      return type;
  }
}


static void ss_iac_message_parser (iac_arguments_t *arguments, gpointer user_data)
{
  gint mess_number;
  gchar *app_temp;
  gchar *con_temp;
  const iac_value_t *val = NULL;

  mess_number = iac_arguments_size (arguments);
  if ((mess_number != 3) && (mess_number != 2))
  {
    g_print ("Parse failure.\n");
    controller = NULL;
    return;
  }
  
  g_print ("%s: %d, mess_number: %d\n", __FUNCTION__, __LINE__, mess_number);
  controller = ss_controller_new ();
  
  val = iac_arguments_get_arg (arguments, 0);
  app_temp = g_strdup (val->value.s);
  controller->applicant = ss_parser_get_applicant (app_temp);
  g_free (app_temp);
  
  val = iac_arguments_get_arg (arguments, 1);
  con_temp = g_strdup (val->value.s);
  controller->type = ss_parser_get_control_type (con_temp);
  g_print ("%s: %d, mess_number: %d\n", __FUNCTION__, __LINE__, mess_number);

  if (mess_number == 3) {
    val = iac_arguments_get_arg (arguments, 2);
    
    if (controller->type == CON_LOCATION) {
      controller->value.location = g_strdup (val->value.s);
      g_print ("%s: %d, location: %s\n", __FUNCTION__, __LINE__, controller->value.location);
    }
    else if (controller->type == CON_VOLUME) {
      g_print ("%s: %d\n", __FUNCTION__, __LINE__);
      controller->value.double_float = atof (val->value.s);
    }
    else {
      g_print ("%s: %d, mess_number: %d\n", __FUNCTION__, __LINE__, mess_number);
      controller->value.double_float = 0;
    }
  }
}


static gboolean  ss_iac_application_handler (const gchar        *interface,
                                             const gchar        *method,
                                             iac_arguments_t    *arguments,
                                             gpointer           user_data)
{
  if (controller)
    ss_controller_free (controller);   //free the previous controller when the new message is coming;
  
  ss_iac_message_parser (arguments, NULL);
  if (!controller)
    return FALSE;
  
  switch (controller->type) {
    case CON_LOCATION: {
      if (!is_playing) {
        is_playing = TRUE;
        g_thread_create ((GThreadFunc )ss_gst_player, controller, FALSE, NULL);
      }
      break;
    }
    
    case CON_MUTE: {
      if (is_playing) {
        ss_gst_volume_set_num (0);
      }
      break;
    }
    
    case CON_STOP: {
      if (is_playing) {
        ss_gst_stop ();
      }
      break;
    }
    
    case CON_VOLUME: {
      if (is_playing) {
        ss_gst_volume_set_shift (controller->value.double_float);
      }
      break;
    }
    
    case CON_REMUTE: {
      if (is_playing) {
        ss_gst_remute ();
      }
      break;
    }
    
    case CON_PAUSE: {
      if (is_playing)
        ss_gst_pause ();
      break;
    }
    
    case CON_CONTINUE: {
      if (is_playing) {
        g_print ("%s: %d\n", __FUNCTION__, __LINE__);
        ss_gst_continue ();
      }
      break;
    }
    
    default:
      break;
  }
  return IAC_OK;
}


gboolean ss_iac_initialize (void)
{
  context = iac_initialize();
  if (context == NULL)
  {
    g_print("%s(): iac_context initialization failure\n",
            __FUNCTION__);
            
    return IAC_ERROR;
  }

  
  if (iac_register_service(context,
      SS_IAC_SERVICE) == IAC_ERROR)
  {
    g_print("%s(): cannot register service\n",
            __FUNCTION__);
            
    return IAC_ERROR;
  }
  
  if (iac_register_listener (context,
                             SS_IAC_OBJECT,
                             SS_IAC_INTERFACE,
                             (iac_listener_func)ss_iac_application_handler,
                             NULL) != IAC_OK)
  {
    iac_deinitialize(context);
    return IAC_ERROR;
  }  
  
  return IAC_OK;
}

void ss_iac_finalize (void)
{
  if (context)
  {
    iac_unregister_listener (context, SS_IAC_OBJECT, SS_IAC_INTERFACE);
    iac_deinitialize(context);
  }
  
  return;
}


SSMessParser *ss_iac_get_controller (void)
{
  return controller;
}


